from datetime import datetime, timedelta

from . import errors
import pytest
import time


class TestGeneral:
    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.general
    def test_stores_credentials(self, connector, good_credentials):
        assert connector.credentials == good_credentials

    def test_env_has_object_id(self, good_credentials):
        assert "object_id" in good_credentials


class TestAuth:
    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.authenticate
    def test_returns_bool(self, good_authentication):
        results = good_authentication
        assert type(results) is bool

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.authenticate
    def test_returns_true(self, good_authentication):
        results = good_authentication
        assert results

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.authenticate
    def test_no_credentials_raises_authentication_error(self, connector_class):
        _connector = connector_class.Connector(None)
        with pytest.raises(errors.AuthenticationError):
            _connector.authenticate()


class TestGetObjects:
    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns(self, get_good_objects):
        # Since we are using a test account, we should have made some objects when we made the account
        assert get_good_objects

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns_list(self, get_good_objects):
        assert type(get_good_objects) is list

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns_nonempty_list(self, get_good_objects):
        assert len(get_good_objects) > 0

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns_dicts_in_list(self, get_good_objects):
        objects = get_good_objects
        for obj in objects:
            assert type(obj) is dict

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns_nonempty_dicts_in_list(self, get_good_objects):
        objects = get_good_objects
        for obj in objects:
            assert len(obj) > 0

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns_object_id_in_dict(self, get_good_objects):
        objects = get_good_objects
        for obj in objects:
            assert "object_id" in obj

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns_object_name_in_dict(self, get_good_objects):
        objects = get_good_objects
        for obj in objects:
            assert "object_name" in obj

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns_object_label_in_dict(self, get_good_objects):
        objects = get_good_objects
        for obj in objects:
            assert "object_label" in obj

    @pytest.mark.source
    @pytest.mark.destination
    @pytest.mark.get_objects
    def test_returns_object_group_in_dict(self, get_good_objects):
        objects = get_good_objects
        for obj in objects:
            assert "object_group" in obj

    # # FIXME: make this test an empty account
    # def test_fail_empty_account(self, get_good_objects):
    #     with not pytest.raises(errors.NoObjectsFoundError):
    #         objects = get_good_objects
    #         pass


class TestGetFields:

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_list(self, get_good_fields):
        results = get_good_fields
        assert type(results) is list

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_nonempty_list(self, get_good_fields):
        results = get_good_fields
        assert len(results) > 0

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_dicts_in_list(self, get_good_fields):
        fields = get_good_fields
        for field in fields:
            assert type(field) is dict

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_nonempty_dicts_in_list(self, get_good_fields):
        fields = get_good_fields
        for field in fields:
            assert len(field) > 0

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_field_id_in_dict(self, get_good_fields):
        fields = get_good_fields
        for field in fields:
            assert "field_id" in field

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_field_name_in_dict(self, get_good_fields):
        fields = get_good_fields
        for field in fields:
            assert "field_name" in field

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_field_label_in_dict(self, get_good_fields):
        fields = get_good_fields
        for field in fields:
            assert "field_label" in field

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_data_type_in_dict(self, get_good_fields):
        fields = get_good_fields
        for field in fields:
            assert "data_type" in field

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_returns_size_in_dict(self, get_good_fields):
        fields = get_good_fields
        for field in fields:
            assert "size" in field

    @pytest.mark.source
    @pytest.mark.get_fields
    def test_bad_object_id_throws_error(self, bad_object_id, connector):
        with pytest.raises(errors.BadObjectIDError):
            connector.authenticate()
            connector.get_fields(bad_object_id)


class TestGetData:
    @pytest.mark.source
    @pytest.mark.get_data
    def test_returns(self, get_good_data_small):
        # Since we are using a test account, we should have made some data for the selected object_id
        data = get_good_data_small
        assert data

    @pytest.mark.source
    @pytest.mark.get_data
    def test_returns_dict(self, get_good_data_small):
        data = get_good_data_small
        assert type(data) is dict

    @pytest.mark.source
    @pytest.mark.get_data
    def test_returns_data_in_dict(self, get_good_data_small):
        data = get_good_data_small
        assert "data" in data

    @pytest.mark.source
    @pytest.mark.get_data
    def test_returns_data_as_list(self, get_good_data_small):
        data = get_good_data_small
        assert type(data["data"]) is list

    @pytest.mark.source
    @pytest.mark.get_data
    def test_returns_nonempty_list(self, get_good_data_small):
        data = get_good_data_small
        assert len(data["data"]) > 0

    @pytest.mark.source
    @pytest.mark.get_data
    def test_returns_dicts_in_list(self, get_good_data_small):
        data = get_good_data_small
        for obj in data["data"]:
            assert type(obj) is dict

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_by_field_id(self, good_object_id, get_three_good_field_ids, connector):
        connector.authenticate()
        data = connector.get_data(good_object_id, get_three_good_field_ids, n_rows=5)
        assert len(data["data"][0].keys()) == len(get_three_good_field_ids)

    @pytest.mark.source
    @pytest.mark.get_data
    def test_bad_field_id_throws_error(self, good_object_id, bad_field_ids, connector):
        with pytest.raises(errors.BadFieldIDError):
            connector.authenticate()
            connector.get_data(good_object_id, bad_field_ids)

    # @pytest.mark.get_fields
    # def test_fail_filter_data_type_throws_error(self, good_object_id, get_all_good_field_ids, connector):
    #     with pytest.raises(errors.FilterDataTypeError):
    #         connector.authenticate()
    #         connector.get_data(good_object_id, get_all_good_field_ids,)

    @pytest.mark.source
    @pytest.mark.get_data
    def test_returns_nonempty_dicts_in_list(self, get_good_data_small):
        data = get_good_data_small
        for obj in data["data"]:
            assert len(obj) > 0

    @pytest.mark.source
    @pytest.mark.get_data
    def test_returns_next_page_flag(self, get_good_data_small):
        data = get_good_data_small
        assert "next_page" in data

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_by_custom_date_returns(self, good_object_id, get_three_good_field_ids, connector,
                                            filtered_column_nm):
        connector.authenticate()
        data = connector.get_data(good_object_id,
                                  get_three_good_field_ids,
                                  filters={
                                      'filtered_column_nm': filtered_column_nm,
                                      'start_selection_nm': "Custom Date",
                                      'end_selection_nm': "Custom Date",
                                      'start_value_txt': "1941-03-26",
                                      'end_value_txt': "2000-01-01",
                                      'timezone_offset_nbr': "1"
                                  })
        assert data

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_by_custom_date_works(self,
                                          good_object_id,
                                          get_three_good_field_ids,
                                          connector,
                                          filtered_column_nm,
                                          filtered_column_date_format,
                                          custom_date_start_val,
                                          custom_date_end_val):
        if filtered_column_nm not in get_three_good_field_ids:
            get_three_good_field_ids.append(filtered_column_nm)
        else:
            get_three_good_field_ids
        connector.authenticate()
        data = connector.get_data(good_object_id,
                                  get_three_good_field_ids,
                                  filters={
                                      'filtered_column_nm': filtered_column_nm,
                                      'start_selection_nm': "Custom Date",
                                      'end_selection_nm': "Custom Date",
                                      'start_value_txt': custom_date_start_val,
                                      'end_value_txt': custom_date_end_val,
                                      'timezone_offset_nbr': "1"
                                  },
                                  n_rows=5)
        start = datetime.strptime(custom_date_start_val, "%Y-%m-%d")
        end = datetime.strptime(custom_date_end_val, "%Y-%m-%d")
        for row in data["data"]:
            assert start < datetime.strptime(row[filtered_column_nm], filtered_column_date_format)
            assert datetime.strptime(row[filtered_column_nm], filtered_column_date_format) < end

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_by_today_returns(self, good_object_id, get_three_good_field_ids, connector, filtered_column_nm):
        connector.authenticate()
        data = connector.get_data(good_object_id,
                                  get_three_good_field_ids,
                                  filters={
                                      'filtered_column_nm': filtered_column_nm,
                                      'start_selection_nm': "Custom Date",
                                      'end_selection_nm': "Today",
                                      'start_value_txt': "1941-03-26",
                                      'end_value_txt': "",
                                      'timezone_offset_nbr': "1"
                                  })
        assert data

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_by_today_works(self,
                                    good_object_id,
                                    get_three_good_field_ids,
                                    connector,
                                    filtered_column_nm,
                                    filtered_column_date_format,
                                    custom_date_start_val,
                                    custom_date_end_val):
        if filtered_column_nm not in get_three_good_field_ids:
            get_three_good_field_ids.append(filtered_column_nm)
        else:
            get_three_good_field_ids
        connector.authenticate()
        data = connector.get_data(good_object_id,
                                  get_three_good_field_ids,
                                  filters={
                                      'filtered_column_nm': filtered_column_nm,
                                      'start_selection_nm': "Custom Date",
                                      'end_selection_nm': "Today",
                                      'start_value_txt': custom_date_start_val,
                                      'end_value_txt': custom_date_end_val,
                                      'timezone_offset_nbr': "1"
                                  },
                                  n_rows=5)
        start = datetime.strptime(custom_date_start_val, "%Y-%m-%d")
        end = datetime.utcnow() - timedelta(days=0, hours=int(1))
        for row in data["data"]:
            assert start < datetime.strptime(row[filtered_column_nm], filtered_column_date_format)
            assert datetime.strptime(row[filtered_column_nm], filtered_column_date_format) < end

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_by_yesterday_returns(self, good_object_id, get_three_good_field_ids, connector,
                                          filtered_column_nm):
        connector.authenticate()
        data = connector.get_data(good_object_id,
                                  get_three_good_field_ids,
                                  filters={
                                      'filtered_column_nm': filtered_column_nm,
                                      'start_selection_nm': "Custom Date",
                                      'end_selection_nm': "Yesterday",
                                      'start_value_txt': "1941-03-26",
                                      'end_value_txt': "",
                                      'timezone_offset_nbr': "1"
                                  })
        assert data

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_by_yesterday_works(self,
                                        good_object_id,
                                        get_three_good_field_ids,
                                        connector,
                                        filtered_column_nm,
                                        filtered_column_date_format,
                                        custom_date_start_val,
                                        custom_date_end_val):
        if filtered_column_nm not in get_three_good_field_ids:
            get_three_good_field_ids.append(filtered_column_nm)
        connector.authenticate()
        data = connector.get_data(good_object_id,
                                  get_three_good_field_ids,
                                  filters={
                                      'filtered_column_nm': filtered_column_nm,
                                      'start_selection_nm': "Custom Date",
                                      'end_selection_nm': "Yesterday",
                                      'start_value_txt': custom_date_start_val,
                                      'end_value_txt': custom_date_end_val,
                                      'timezone_offset_nbr': "1"
                                  },
                                  n_rows=5)
        start = datetime.strptime(custom_date_start_val, "%Y-%m-%d")
        end = datetime.utcnow() - timedelta(days=1, hours=int(1))
        for row in data["data"]:
            assert start < datetime.strptime(row[filtered_column_nm], filtered_column_date_format)
            assert datetime.strptime(row[filtered_column_nm], filtered_column_date_format) < end

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_no_start_selection(self,
                                        good_object_id,
                                        get_three_good_field_ids,
                                        connector,
                                        filtered_column_nm,
                                        filtered_column_date_format,
                                        custom_date_end_val):
        if filtered_column_nm not in get_three_good_field_ids:
            get_three_good_field_ids.append(filtered_column_nm)
        connector.authenticate()
        data = connector.get_data(good_object_id,
                                  get_three_good_field_ids,
                                  filters={
                                      'filtered_column_nm': filtered_column_nm,
                                      'start_selection_nm': None,
                                      'end_selection_nm': "Yesterday",
                                      'start_value_txt': None,
                                      'end_value_txt': custom_date_end_val,
                                      'timezone_offset_nbr': "1"
                                  },
                                  n_rows=5)
        end = datetime.utcnow() - timedelta(days=1, hours=int(1))
        for row in data["data"]:
            assert datetime.strptime(row[filtered_column_nm], filtered_column_date_format) < end

    @pytest.mark.source
    @pytest.mark.get_data
    def test_filters_no_end_selection(self,
                                      good_object_id,
                                      get_three_good_field_ids,
                                      connector,
                                      filtered_column_nm,
                                      filtered_column_date_format,
                                      custom_date_start_val):
        if filtered_column_nm not in get_three_good_field_ids:
            get_three_good_field_ids.append(filtered_column_nm)
        connector.authenticate()
        data = connector.get_data(good_object_id,
                                  get_three_good_field_ids,
                                  filters={
                                      'filtered_column_nm': filtered_column_nm,
                                      'start_selection_nm': "Custom Date",
                                      'end_selection_nm': None,
                                      'start_value_txt': custom_date_start_val,
                                      'end_value_txt': None,
                                      'timezone_offset_nbr': "1"
                                  },
                                  n_rows=5)
        start = datetime.strptime(custom_date_start_val, "%Y-%m-%d")
        for row in data["data"]:
            assert start < datetime.strptime(row[filtered_column_nm], filtered_column_date_format)


class TestGetMetaData:
    @pytest.mark.destination
    @pytest.mark.get_metadata
    def test_returns_column_type_flg(self, get_good_meta_data):
        data = get_good_meta_data
        assert "column_type_flg" in data

    @pytest.mark.destination
    @pytest.mark.get_metadata
    def test_returns_size_flg(self, get_good_meta_data):
        data = get_good_meta_data
        assert "size_flg" in data

    @pytest.mark.destination
    @pytest.mark.get_metadata
    def test_returns_new_object_regex(self, get_good_meta_data):
        data = get_good_meta_data
        assert "new_object_regex" in data

    @pytest.mark.destination
    @pytest.mark.get_metadata
    def test_returns_size_regex(self, get_good_meta_data):
        data = get_good_meta_data
        assert "size_regex" in data

    @pytest.mark.destination
    @pytest.mark.get_metadata
    def test_returns_data_types(self, get_good_meta_data):
        data = get_good_meta_data
        assert "data_types" in data


class TestLoadData:
    @pytest.mark.destination
    @pytest.mark.load_data
    def test_load_data_append(self, connector, good_object_id, mapping_object, _append, five_column_100_row_dataset):
        try:
            connector.load_data(five_column_100_row_dataset,
                                good_object_id,
                                mapping_object,
                                _append,
                                batch_number=1,
                                total_batches=1)
        except Exception as e:
            assert False, f"Appending 100 rows of 5-column data raised an exception {e}"

    @pytest.mark.destination
    @pytest.mark.load_data
    def test_load_data_replace(self, connector, good_object_id, mapping_object, _replace, five_column_100_row_dataset):
        try:
            connector.load_data(five_column_100_row_dataset,
                                good_object_id,
                                mapping_object,
                                _replace,
                                batch_number=1,
                                total_batches=1)
        except Exception as e:
            assert False, f"Replacing 100 rows of 5-column data raised an exception {e}"

    @pytest.mark.destination
    @pytest.mark.load_data
    def test_load_data_create_new_table(self,
                                        connector,
                                        bad_object_id,
                                        mapping_object,
                                        _replace,
                                        five_column_100_row_dataset):
        # FIXME: a potential issue here is that i do not have object name validation for "_bad" on an object_id
        connector.load_data(five_column_100_row_dataset,
                            bad_object_id,
                            mapping_object,
                            _replace,
                            batch_number=1,
                            total_batches=1)
        connector.authenticate()
        objects = connector.get_objects()
        assert bad_object_id in [obj["object_id"] for obj in objects]

    @pytest.mark.destination
    @pytest.mark.load_test_slow
    def test_load_large_data(self,
                             load_data):
        start = time.time()
        assert load_data
        end = time.time()
        elapsed_time = end - start
        print(f"Time elapsed to load {load_data.param} GB: " + str(elapsed_time))
        assert elapsed_time < 10 * 60



import os
import sys
import pytest
from dotenv import load_dotenv
from os import environ
import random
import string
import shutil
from faker.providers.person.en import Provider
import numpy as np
import pandas as pd

GB = 1024 * 1024 * 1024
MB = 1024 * 1024


def pytest_addoption(parser):
    """
    adds the option to write --file=<your path to connector.py>
    """
    parser.addoption(
        "--source", action="store_true", default=False, help="run source tests"
    )
    parser.addoption(
        "--destination", action="store_true", default=False, help="run destination tests"
    )


@pytest.fixture(scope="session")
def root_dir():
    return os.path.dirname(os.path.abspath(".env.credentials"))


def add_root_dir_to_path(root_dir):
    path = os.path.abspath(os.path.join(os.path.dirname(__file__), root_dir))
    # print(path)
    if path not in sys.path:
        sys.path.insert(1, path)


@pytest.fixture(scope="session", autouse=True)
def copy_ini_to_root(root_dir):
    add_root_dir_to_path(root_dir)
    shutil.copy(os.path.join(os.path.dirname(__file__), "pytest.txt"),
                os.path.abspath("pytest.ini"))


@pytest.fixture(scope="session")
def module_name(pytestconfig):
    return pytestconfig.getoption("lib-path")


@pytest.fixture(scope="session",
                autouse=True)
def connector_class(root_dir):
    """ This takes in the connector file name and tries to import it """
    try:
        add_root_dir_to_path(root_dir)
        import src.connector as _conn
        return _conn
    except Exception:
        raise Exception("Error: cannot locate Connector class in session <session name>")


@pytest.fixture(scope="session")
def env_file(pytestconfig):
    return ".env.credentials"


def pytest_configure(config):
    config.addinivalue_line("markers", "source: mark test as a source function")


# def pytest_collection_modifyitems(config, items):
# """ This function makes pytest skip the source or destination tests, depending if the --source or --destination
# flag is not present. """
# if not config.getoption("--source"):
#     # makes the test print the reason the test was skipped
#     skip_source = pytest.mark.skip(reason="need --source option to run")
#     for item in items:
#         if "source" in item.keywords:
#             item.add_marker(skip_source)
#
# if not config.getoption("--destination"):
#     skip_destination = pytest.mark.skip(reason="need --destination option to run")
#     for item in items:
#         if "destination" in item.keywords:
#             item.add_marker(skip_destination)


@pytest.fixture(scope="session")
def env(env_file):
    load_dotenv(env_file)
    return environ


@pytest.fixture(scope="session")
def good_object_id(env):
    return env["OBJECT_ID"]


@pytest.fixture(scope="session")
def bad_object_id(env):
    return env["OBJECT_ID"] + "_bad"


@pytest.fixture(scope="session")
def good_credentials(env):
    """ This takes the credentials in the env file and converts it to a dictionary """
    keys = list(env.keys())
    values = list(env.values())
    res = {keys[i].lower(): values[i] for i in range(len(keys))}
    if "USER_NAME" in env.keys():
        res["username"] = env["USER_NAME"]
    return res


@pytest.fixture(scope="session")
def bad_credentials(env):
    """ This has all the keys in the credentials object but those keys are mapped to random integers """
    keys = list(env.keys())
    res = {keys[i].lower(): str(i) for i in range(len(keys))}
    return res


@pytest.fixture(scope="session")
def filtered_column_nm(env):
    return env["FILTERED_COLUMN_NM"]


@pytest.fixture(scope="session")
def filtered_column_date_format(env):
    return env["FILTERED_COLUMN_DATE_FORMAT"]


@pytest.fixture(scope="session")
def earliest_date():
    return "0000-01-01"


@pytest.fixture(scope="session")
def custom_date_start_val():
    return "1941-03-26"


@pytest.fixture(scope="session")
def custom_date_end_val():
    return "2000-01-01"


@pytest.fixture(scope="session")
def connector(connector_class, good_credentials):
    connector = connector_class.Connector(good_credentials)
    return connector


@pytest.fixture(scope="session")
def good_authentication(connector):
    results = connector.authenticate()
    return results


@pytest.fixture(scope="session")
def get_good_objects(connector):
    connector.authenticate()
    objects = connector.get_objects()
    return objects


@pytest.fixture(scope="session")
def get_good_fields(good_object_id, connector):
    connector.authenticate()
    results = connector.get_fields(good_object_id)
    return results


@pytest.fixture(scope="session")
def get_all_good_field_ids(good_object_id, connector):
    connector.authenticate()
    results = connector.get_fields(good_object_id)
    return [result["field_id"] for result in results]


@pytest.fixture(scope="session")
def get_three_good_field_ids(good_object_id, connector):
    connector.authenticate()
    results = connector.get_fields(good_object_id)
    field_ids = [result["field_id"] for result in results]
    return field_ids[:3]


@pytest.fixture(scope="session")
def bad_field_ids(get_all_good_field_ids):
    FIELD_ID_LEN = 10

    # create a random 10-char string to add to get_good_field_ids
    letters = string.ascii_lowercase
    field_id = ''.join(random.choice(letters) for i in range(FIELD_ID_LEN))

    # if the random string happens to be a valid field id for this object, try again
    num_tries = 0
    while field_id in get_all_good_field_ids:
        num_tries += 1

        # if it tries 15 times and is still a valid field id, throw an exception
        if num_tries > 15:
            raise Exception
        field_id = ''.join(random.choice(letters) for i in range(FIELD_ID_LEN))

    # changing the last valid field id for the invalid one
    field_ids = get_all_good_field_ids
    field_ids[-1] = field_id
    return field_ids


@pytest.fixture(scope="session")
def bad_filters_data_type(env):
    # ? maybe have a field called "FILTERED_COLUMN_NM_WRONG_DATA_TYPE"?
    return {
        "filtered_column_nm": env["FILTERED_COLUMN_NM"],
        "start_selection_nm": env["FILTERED_SELECTION_NM"],
        "end_selection_nm": env["END_SELECTION_NM"],
        "start_value_txt": env["START_VALUE_TXT"],
        "end_value_txt": env["END_VALUE_TXT"],
        "timezone_offset_nbr": int(env["TIMEZONE_OFFSET_NBR"])
    }


@pytest.fixture(scope="session")
def get_good_data_small(connector, good_object_id, get_all_good_field_ids):
    connector.authenticate()
    # Since we are using a test account, we should have made some data for the selected object_id
    data = connector.get_data(object_id=good_object_id, field_ids=get_all_good_field_ids, n_rows=5)
    return data


@pytest.fixture(scope="session")
def get_good_meta_data(connector):
    connector.authenticate()
    meta = connector.get_metadata()
    return meta


@pytest.fixture(scope="session")
def batch_size():
    return 2.5 * MB


def random_names(name_type, size):
    """
    Generate n-length ndarray of person names.
    name_type: a string, either first_names or last_names
    """
    names = getattr(Provider, name_type)
    return np.random.choice(names, size=size)


def random_genders(size, p=None):
    """Generate n-length ndarray of genders."""
    if not p:
        # default probabilities
        p = (0.49, 0.49, 0.01, 0.01)
    gender = ("M", "F", "O", "")
    return np.random.choice(gender, size=size, p=p)


def random_dates(start, end, size):
    """
    Generate random dates within range between start and end.
    """
    # Unix timestamp is in nanoseconds by default, so divide it by
    # 24*60*60*10**9 to convert to days.
    divide_by = 24 * 60 * 60 * 10 ** 9
    start_u = start.value // divide_by
    end_u = end.value // divide_by
    return pd.to_datetime(np.random.randint(start_u, end_u, size), unit="D")


def random_numbers(start, end, size):
    """
    Generate random numbers within range between start and end.
    """
    return np.random.randint(start, end, size)


@pytest.fixture(scope="session")
def five_column_100_row_dataset():
    size = 100
    df = pd.DataFrame(columns=['First', 'Last', 'Gender', 'Birthdate'])
    df['First'] = random_names('first_names', size)
    df['Last'] = random_names('last_names', size)
    df['Gender'] = random_genders(size)
    df['BirthDate'] = random_dates(start=pd.to_datetime('1940-01-01'), end=pd.to_datetime('2022-01-01'), size=size)
    df['StartDate'] = random_dates(start=pd.to_datetime('1940-01-01'), end=pd.to_datetime('2022-01-01'), size=size)
    return to_dc_format(df)


@pytest.fixture(scope="session", params=[1, 10, 100, 1000])
def batch():
    size = 100
    df = pd.DataFrame(columns=['First', 'Last', 'Gender', 'Birthdate'])
    df['First'] = random_names('first_names', size)
    df['Last'] = random_names('last_names', size)
    df['Gender'] = random_genders(size)
    df['BirthDate'] = random_dates(start=pd.to_datetime('1940-01-01'), end=pd.to_datetime('2022-01-01'), size=size)
    df['StartDate'] = random_dates(start=pd.to_datetime('1940-01-01'), end=pd.to_datetime('2022-01-01'), size=size)
    print(sys.getsizeof(to_dc_format(df)))
    return to_dc_format(df)


@pytest.fixture(scope="session", params=('size', [1, 10, 100, 1000]))
def load_data(connector, batch, good_object_id, mapping_object, _replace, size, batch_size):
    total_batches = (size * GB) / batch_size
    batch_number = 1
    rows_loaded = 0
    connector.load_data(batch, good_object_id, mapping_object, _replace, batch_number, total_batches)

    while batch_number <= total_batches:
        rows_loaded += len(batch)
        print(f"Loaded {rows_loaded} rows out of {total_batches * len(batch)}")
        batch_number += 1
        connector.load_data(batch, good_object_id, mapping_object, _replace, batch_number, total_batches)

    print(f"Loaded {rows_loaded} rows out of {total_batches * len(batch)}.\nTotal size uploaded: {size * GB}")
    return True


@pytest.fixture(scope="session")
def mapping_object():
    return [
        {'source_field_id': "First", 'destination_field_id': "newfirst", 'datatype': "varchar(128)", 'size': None},
        {'source_field_id': "Last", 'destination_field_id': "newlast", 'datatype': "varchar(256)", 'size': None},
        {'source_field_id': "Gender", 'destination_field_id': "newgender", 'datatype': "varchar(128)", 'size': None},
        {'source_field_id': "BirthDate", 'destination_field_id': "newbirthdate", 'datatype': "date", 'size': None},
        {'source_field_id': "StartDate", 'destination_field_id': "newstartdate", 'datatype': "date", 'size': None}
    ]


def ten_column_dataset(num_rows):
    size = num_rows
    df = pd.DataFrame(columns=['First', 'Last', 'Gender', 'Birthdate'])
    df['First'] = random_names('first_names', size)
    df['Last'] = random_names('last_names', size)
    df['Gender'] = random_genders(size)
    df['BirthDate'] = random_dates(start=pd.to_datetime('1940-01-01'), end=pd.to_datetime('2022-01-01'), size=size)
    df['StartDate'] = random_dates(start=pd.to_datetime('1940-01-01'), end=pd.to_datetime('2022-01-01'), size=size)
    df['FavoriteNum'] = random_numbers(start=0, end=1000000, size=size)
    df['LeastFavoriteNum'] = random_numbers(start=0, end=1000000, size=size)
    df['NumberOfTimesSaidHello'] = random_numbers(start=0, end=1000000, size=size)
    df['SaddestAge'] = random_numbers(start=0, end=1000000, size=size)
    df['NumberOfPushupsDoneEver'] = random_numbers(start=0, end=1000000, size=size)
    return to_dc_format(df)


def to_dc_format(df):
    df = df.to_dict("records")
    return df


@pytest.fixture(scope="session")
def _append():
    return 0


@pytest.fixture(scope="session")
def _replace():
    return 1

